@@ -1,4 +1,6 @@
 package Text::Corpus::Summaries::Wikipedia;
+
+use utf8;
 use strict;
 use warnings;
 use Carp;
@@ -15,12 +17,12 @@ use Digest::MD5 qw(md5_hex);
 use Date::Manip;
 use File::Copy;
 use XML::Code;
-#use Data::Dump qw(dump);
+use Data::Dump qw(dump);
 
 BEGIN {
     use Exporter ();
     use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-    $VERSION     = '0.21';
+    $VERSION     = '0.22';
     @ISA         = qw(Exporter);
     @EXPORT      = qw();
     @EXPORT_OK   = qw();
@@ -28,7 +30,7 @@ BEGIN {
 }
 
 =head1 NAME
-
+ 
 C<Text::Corpus::Summaries::Wikipedia> - Creates corpora for summarization testing.
 
 =head1 SYNOPSIS
@@ -41,11 +43,11 @@ C<Text::Corpus::Summaries::Wikipedia> - Creates corpora for summarization testin
 
 =head1 DESCRIPTION
 
-C<Text::Corpus::Summaries::Wikipedia> creates corpora for summarization testing
+C<Text::Corpus::Summaries::Wikipedia> creates corpora for single document summarization testing
 using the featured articles of various Wikipedias.
 
 A criterion for an article in a Wikipedia to be I<featured> is that it
-have a well written lead section, or introduction; thus, the featured articles
+have a well written lead section, or introduction. So the featured articles
 of a Wikipedia can make an excellent corpus for testing single document summarization methods.
 This module creates a corpus from the featured articles of a Wikipedia by fetching
 and saving their content as HTML, text, and XML, with the appropriate sections
@@ -76,8 +78,18 @@ class with the following parameters:
 
 C<languageCode> is the language code of the Wikipedia from which the
 corpus of featured articles are to be created. The supported language codes are
-C<af>:Afrikaans, C<ar>:Arabic, C<bg>:Bulgarian, C<ca>:Catalan, C<cs>:Czech, C<de>:German, C<el>:Greek, C<en>:English, C<eo>:Esperanto, C<es>:Spanish, C<eu>:Basque, C<fa>:Persian, C<fi>:Finnish, C<fr>:French, C<he>:Hebrew, C<hr>:Croatian, C<hu>:Hungarian, C<id>:Indonesian, C<it>:Italian, C<ja>:Japanese, C<ka>:Georgian, C<ko>:Korean, C<ml>:Malayalam, C<mr>:Marathi, C<ms>:Malay, C<nl>:Dutch, C<nn>:Norwegian, C<no>:Norwegian, C<pl>:Polish, C<pt>:Portuguese, C<ro>:Romanian, C<ru>:Russian, C<sh>:Serbocroatian, C<sk>:Slovak, C<sl>:Slovenian, C<sr>:Serbian, C<sv>:Swedish, C<th>:Thai, C<tr>:Turkish, C<vi>:Vietnamese, and C<zh>:Chinese.
-The default is C<en>.
+C<af>:Afrikaans, C<ar>:Arabic, C<az>:Azerbaijani, C<bg>:Bulgarian, C<bs>:Bosnian, C<ca>:Catalan, 
+C<cs>:Czech, C<de>:German, C<el>:Greek, C<en>:English, C<eo>:Esperanto, C<es>:Spanish, C<eu>:Basque, 
+C<fa>:Persian, C<fi>:Finnish, C<fr>:French, C<he>:Hebrew, C<hr>:Croatian, C<hu>:Hungarian, 
+C<id>:Indonesian, C<it>:Italian, C<ja>:Japanese, C<jv>:Javanese, C<ka>:Georgian, C<kk>:Kazakh, 
+C<km>:Khmer, C<ko>:Korean, C<li>:Limburgish, C<lv>:Latvian, C<ml>:Malayalam, C<mr>:Marathi, 
+C<ms>:Malay, C<mzn>:Mazandarani, C<nl>:Dutch, C<nn>:Norwegian (Nynorsk), C<no>:Norwegian (Bokm?l), 
+C<pl>:Polish, C<pt>:Portuguese, C<ro>:Romanian, C<ru>:Russian, C<sh>:Serbo-Croatian, 
+C<simple>:Simple English, C<sk>:Slovak, C<sl>:Slovenian, C<sr>:Serbian, C<sv>:Swedish, 
+C<sw>:Swahili, C<ta>:Tamil, C<th>:Thai, C<tl>:Tagalog, C<tr>:Turkish, C<tt>:Tatar, 
+C<uk>:Ukrainian, C<ur>:Urdu, C<vi>:Vietnamese, C<vo>:Volap?k, and C<zh>:Chinese.
+If the language code is C<all>, then the corpus for each supported language is
+created (which takes a long time). The default is C<en>.
 
 =item C<corpusDirectory>
 
@@ -111,7 +123,7 @@ sub new
 
   # timeOfLast fetch is the time of the last page fetch.
   $Self->{timeOfLastFetch} = 0;
-  $Self->{minSecondsBetweenFetches} = 8;
+  $Self->{minSecondsBetweenFetches} = 2;
 
   # make the user agent
   $Self->{userAgent} = LWP::UserAgent->new;
@@ -149,7 +161,7 @@ sub new
   # get the language code.
   $Parameters{languageCode} = 'en' unless exists $Parameters{languageCode};
   $Self->{languageCode} = lc $Parameters{languageCode};
-  $Self->{languageCode} = $1 if ($Self->{languageCode} =~ /^([a-z]+)/);
+  $Self->{languageCode} = $1 if ($Self->{languageCode} =~ /^([a-z\-]+)/);
   unless (defined $Self->_getFeatureArticleUrlAndXPathExpression ())
   {
     croak "Language with code '" . $Parameters{languageCode} . "' is not supported.\n";
@@ -680,6 +692,8 @@ sub getParsedArticlePage # (htmlFile => '...')
   close $filePtr;
 
   #$htmlParser->parse_file ($htmlFile);
+  
+  my $xpathTitle = '/html/body//h1[@id="firstHeading"]';
 
   # xpathBodyContent is the base xpath to the content of the article.
   # my $xpathBodyContent = '/html/body/div[@id="globalWrapper"]/div[@id="column-content"]/div[@id="content"]/div[@id="bodyContent"]';
@@ -687,7 +701,9 @@ sub getParsedArticlePage # (htmlFile => '...')
   my $xpathBodyContent = '//div[@id="bodyContent"]';
 
   # xpathContentParagraphs is the xpath to the paragraphs we want plus a little more that will be removed later.
-  my $xpathContentParagraphs = $xpathBodyContent . '/p';
+  my $xpathContentParagraphs = $xpathBodyContent . '//p';
+  #my $xpathContentParagraphs = '/html/body/div/div[@id="bodyContent"]/div/p';
+  
 
   # xpathToc is the xpath to the table of contents; paragraphs before this are the summary.
   #my $xpathToc = $xpathBodyContent . '/table[@id="toc"]';
@@ -701,31 +717,34 @@ sub getParsedArticlePage # (htmlFile => '...')
 
   # xpathHeaders is a list of all the xpath headers h1 to h9.
   my @xpathHeaders;
-  for (my $i = 1; $i < 10; $i++)
+  for (my $i = 1; $i < 9; $i++)
   {
-    $xpathHeaders[$i-1] = $xpathBodyContent . '/h' . $i;
+    $xpathHeaders[$i-1] = $xpathBodyContent . '//h' . ($i + 1) . '/span[@class="mw-headline"]';
   }
 
   # xpathHeaders is the xpath expression to find any header h1 to h9.
   my $xpathHeaders = join ('|', @xpathHeaders);
 
   # xpaths is the xpath expression for all the content we want to find.
-  my $xpaths = join ('|', $xpathContentParagraphs, $xpathToc, $xpathHeaders, $xpathReferences);
+  my $xpaths = join ('|', $xpathTitle, $xpathContentParagraphs, $xpathToc, $xpathHeaders, $xpathReferences);
 
   # find all the nodes in the html containing content we want.
   my @contentNodes = $htmlParser->findnodes ($xpaths);
-
+  
   # convert the nodes found into a list where each entry in the list is of the
   # form [type, content]. type is either 'paragraph', 'headerN', 'toc', or 'references'
   # and content is the text content of the node.
   my @lines;
   foreach my $node (@contentNodes)
   {
+    # get the tag of the node.
+    my $tag = $node->tag();
+    
     # default type is a paragraph.
     my $type = 'paragraph';
     my $value = $node->findvalue ('.');
 
-    # the strings returned should be utf8 encoded, if not, make thme so.
+    # the strings returned should be utf8 encoded, if not, make them so.
     unless (Encode::is_utf8 ($value, 1))
     {
       eval { $value = decode_utf8 ($value, 0); };
@@ -736,30 +755,55 @@ sub getParsedArticlePage # (htmlFile => '...')
       }
     }
 
+    # most common is a paragraph.
+    if ($tag eq 'p')
+    {
+      # skip the node if all white space.
+      next if ($value =~ /^\s*$/);
+
+      # store the paragraph.
+      push @lines, ['paragraph', $value];
+      
+      next;
+    }
+    
+    # if hN, it is a header.
+    if ($tag =~ /^h(\d+)$/)
+    {
+      push @lines, ['header' . $1, $value];
+      next;
+    }
+        
+    # if span get the header with it.
+    if ($tag eq 'span')
+    {
+      my $parentNode = $node->parent();
+      my $parentNodeTag = $parentNode->tag();
+      if ($parentNodeTag =~ /^h(\d)/)
+      {
+        my $headerDepth = $1;
+        if ($value =~ /^\s*(Sees\*Also|Notes|References|External\s*Links)\s*$/i)
+        {
+          push @lines, ['references', $value];
+        }
+        else
+        {
+          push @lines, ['header' . $headerDepth, $value];
+        }
+        next;
+      }
+    }
+
     # check for 'toc' table.
-    if ($node->matches ($xpathToc))
+    if ($tag eq 'table')
     {
       $type = 'toc';
       push @lines, [$type, $value];
       next;
     }
 
-    # check if the node is a header.
-    my $headerFound = 0;
-    for (my $i = 1; $i < 10; $i++)
-    {
-      if ($node->matches ($xpathHeaders[$i-1]))
-      {
-        $type = 'header' . $i;
-        $headerFound = $i;
-        push @lines, [$type, $value];
-        last;
-      }
-    }
-    next if $headerFound;
-
-    # check is the node has the list of references.
-    if ($node->matches ($xpathReferences))
+    # check if the node has the list of references.
+    if ($tag eq 'ol')
     {
       $type = 'references';
       push @lines, [$type, $value];
@@ -780,6 +824,17 @@ sub getParsedArticlePage # (htmlFile => '...')
     $lines[$i]->[1] =~ s/[\p{SpacingMark}\p{Whitespace}\x{A0}]+/ /g;
   }
 
+  # find the title, if there is one.
+  my $titleText;
+  for (my $i = 0; $i < @lines; $i++)
+  {
+    if ($lines[$i]->[0] eq 'header1')
+    {
+      $titleText = $lines[$i]->[1];
+      last;
+    }
+  }
+
   # find the index for the 'toc' tag.
   my $tocIndex = -1;
   for (my $i = 0; $i < @lines; $i++)
@@ -810,6 +865,11 @@ sub getParsedArticlePage # (htmlFile => '...')
       splice (@lines, 0, $tocIndex + 1);
     }
   }
+  else
+  {
+    my $logger = Log::Log4perl->get_logger();
+    $logger->logwarn ("Parsing problems, could not find toc in file '$htmlFile'.\n");
+  }
 
   # find the first reference tag and remove all lines after and including it.
   for (my $i = 0; $i < @lines; $i++)
@@ -841,6 +901,14 @@ sub getParsedArticlePage # (htmlFile => '...')
   $xmlDocument->encoding("UTF-8");
   my @xmlStack = ($xmlDocument);
 
+  # add the title.
+  if (defined $titleText)
+  {
+    my $title = XML::Code->new ('title');
+    $xmlStack[-1]->add_child ($title);
+    $title->set_text ($titleText);
+  }
+
   # add the summary.
   my $summary = XML::Code->new ('summary');
   $xmlStack[-1]->add_child ($summary);
@@ -1063,7 +1131,7 @@ sub _getListOfFeaturedArticleUrlsViaUrlAndXpath
 
   # extract out the links for the featured articles from the html.
   my @urls = $htmlParser->findvalues ($xpath);
-
+  
   # remove any titles that have substrings identifying them as not featured articles.
   my $totalUrls = 0;
   for (my $i = 0; $i < @urls; $i++)
@@ -1307,7 +1375,8 @@ sub _getFeaturedArticleLinkXpathPerLanguage
 
   # set the xpath to use to extract the urls of the featured articles.
   # my $xpath = '/html/body/div/div/div[8]/div/ul/li/a/@href';
-  my $xpath = '/html/body/div/div/div/ul/li/a/@href';
+  # my $xpath = '/html/body/div[5]/div[6]/div/ul/li/a/@href';
+  my $xpath = '/html/body/div/div/div[@id="p-lang"]/div[@class="body"]/ul/li/a/@href';
 
   # fetch the page.
   my $fileName = 'featured.html';
@@ -1350,14 +1419,20 @@ sub _getFeaturedArticleLinkXpathPerLanguage
     {
       $featuredArticlesUrls{lc $1} = $url;
     }
+    elsif ($url =~ /^\/\/([^\.]+)?\./)
+    {
+      $featuredArticlesUrls{lc $1} = 'http:'.$url;
+    }
   }
+
   $Self->{featuredArticlesUrls} = \%featuredArticlesUrls;
 
   # get the xpath expression to extract the article links also.
   $Self->_setXpathForFeaturedArticles ();
 
-  # some wikipedias do not list their features articles on main featured articles page.
+  # some wikipedias do not list their features articles on the main featured articles page.
   $Self->{featuredArticlesUrls}{id} = 'http://id.wikipedia.org/wiki/Wikipedia:Artikel_pilihan/Topik';
+  $Self->{featuredArticlesUrls}{simple} = 'http://simple.wikipedia.org/wiki/Wikipedia:Very_good_articles/by_date';
 
   # gotta call delete on HTML::TreeBuilder, or we get a memory leak.
   $htmlParser->delete;
@@ -1379,49 +1454,159 @@ sub _setXpathForFeaturedArticles
   my %featuredArticlesXpath;
 
   # single page featured article wikipedias require only an xpath to extract the featured article urls.
-  $featuredArticlesXpath{af} = '/html/body/div/div/table/tr/td/p//a/@href';
-  $featuredArticlesXpath{ar} = '/html/body/div/div/table/tr/td//a/@href';
-  $featuredArticlesXpath{bg} = '/html/body/div/div/div/table/tr/td/ul/li/a/@href';
-  $featuredArticlesXpath{ca} = '/html/body/div/div/table[3]//a/@href';
-  $featuredArticlesXpath{cs} = '/html/body/div/div/table/tr[2]//a/@href';
-  $featuredArticlesXpath{de} = '/html/body/div/div/table/tr[position()>2]//a/@href';
-  $featuredArticlesXpath{el} = '/html/body/div/div/ol/li/b//a/@href';
-  $featuredArticlesXpath{en} = '/html/body/div/div/table/tr[3]//a/@href';
-  $featuredArticlesXpath{eo} = '/html/body/div/div/table/tr[position()>3]/td/p//a/@href';
-  $featuredArticlesXpath{es} = '/html/body/div/div/table[3]/tr[3]/td//a/@href';
-  $featuredArticlesXpath{eu} = '/html/body/div/div/ul/li/a/@href';
-  $featuredArticlesXpath{fa} = '/html/body/div/div/table[3]/tr[2]/td//a/@href';
-  $featuredArticlesXpath{fi} = '/html/body/div/div/table[position()>3]//a/@href';
-  $featuredArticlesXpath{he} = '/html/body/div/div/table/tr[position()>4]/td//a/@href';
-  $featuredArticlesXpath{hr} = '/html/body/div/div/div/div[2]/table/tr[4]/td//a/@href';
-  $featuredArticlesXpath{hu} = '/html/body/div/div/table[3]/tr/td/p//a/@href';
-  $featuredArticlesXpath{it} = '/html/body/div/div/table/tr[7]/td/dl/dd//a/@href';
-  $featuredArticlesXpath{id} = '/html/body/div/div/table[2]/tr[2]/td//a/@href';
-  $featuredArticlesXpath{ka} = '/html/body/div/div/div/table/tr[position()>2]//a/@href';
-  $featuredArticlesXpath{ko} = '/html/body/div/div/table/tr[3]/td//a/@href';
-  $featuredArticlesXpath{ml} = '/html/body/div/div/table/tr/td[2]//a/@href';
-  $featuredArticlesXpath{mr} = '//*[@id="bodyContent"]//b/a/@href';
-  $featuredArticlesXpath{ms} = '/html/body/div/div/table/tr[3]/td//a/@href';
-  $featuredArticlesXpath{nl} = '/html/body/div/div/table/tr/td/p//a/@href';
-  $featuredArticlesXpath{nn} = '/html/body/div/div/div/div[2]/table/tr[2]/td/table[3]/tr[4]/td//ul//a/@href';
-  $featuredArticlesXpath{no} = '/html/body/div/div/table/tr[position()>1]/td//a/@href';
-  $featuredArticlesXpath{pl} = '/html/body/div/div/table/tr[3]/td//a/@href';
-  $featuredArticlesXpath{pt} = '/html/body/div/div/table/tr/td/dl/dd//a/@href';
-  $featuredArticlesXpath{ro} = '/html/body/div/div/table[2]/tr[3]/td/dl/dd//a/@href';
-  $featuredArticlesXpath{sh} = '/html/body/div/div/div/div/ul/li/b//a/@href';
-  $featuredArticlesXpath{sl} = '/html/body/div/div/table[2]/tr[3]/td//a/@href';
-  $featuredArticlesXpath{sr} = '/html/body/div/div/div/div/table/tr/td/dl/dd//a/@href';
-  $featuredArticlesXpath{sv} = '/html/body/div/div/table/tr/td/table/tr/td/dl/dd/b//a/@href';
-  $featuredArticlesXpath{tr} = '/html/body/div/div/table/tr[3]/td/p//a/@href';
-  $featuredArticlesXpath{vi} = '/html/body/div/div/div/table/tr/td/p//a/@href';
-  $featuredArticlesXpath{zh} = '/html/body/div/div/table[3]//a/@href';
-  $featuredArticlesXpath{sk} = '/html/body/div/div/p//a/@href';
-  $featuredArticlesXpath{th} = '/html/body/div/div/table[2]//a/@href';
-	$featuredArticlesXpath{ja} = '/html/body/div/div/ul/li//a/@href';
+  #$featuredArticlesXpath{af} = '/html/body/div/div/table/tr/td/p//a/@href';
+  $featuredArticlesXpath{af} = '/html/body/div[3]/div[3]/div[4]/table/tr[position()>3]/td/p/a/@href';
+    
+  #$featuredArticlesXpath{ar} = '/html/body/div/div/table/tr/td//a/@href';
+  $featuredArticlesXpath{ar} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr[2]/td/ul/li/a/@href'; 
+
+  $featuredArticlesXpath{az} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td/ol/li/a/@href';
+  
+  # $featuredArticlesXpath{'be-x-old'} = '/html/body/div[3]/div[3]/div[4]/table[4]/tr/td/p/a/@href';
+  
+  $featuredArticlesXpath{bs} = '/html/body/div[3]/div[3]/div[4]/table/tr[2]/td/dl/dd//a/@href';
+  
+  #$featuredArticlesXpath{bg} = '/html/body/div/div/div/table/tr/td/ul/li/a/@href';
+  $featuredArticlesXpath{bg} = '/html/body/div[3]/div[3]/div[4]/div/table[4]/tr/td/ul/li/a/@href';
+  
+  #$featuredArticlesXpath{ca} = '/html/body/div/div/table[3]//a/@href';
+  $featuredArticlesXpath{ca} = '/html/body/div[3]/div[3]/div[4]/table[3]/tr/td//table/tr[2]/td//a/@href';
+  
+  #$featuredArticlesXpath{cs} = '/html/body/div/div/table/tr[2]//a/@href';
+  $featuredArticlesXpath{cs} = '/html/body/div[3]/div[3]/div[4]/table/tr[2]/td/dl/dd/a/@href';
+  
+  #$featuredArticlesXpath{de} = '/html/body/div/div/table/tr[position()>2]//a/@href';
+  $featuredArticlesXpath{de} = '/html/body/div[3]/div[3]/div[4]/table/tr[2]/td/table[2]/tr[position()>2]/td/table[2]/tr/td/p/a/@href';
+  
+  #$featuredArticlesXpath{el} = '/html/body/div/div/ol/li/b//a/@href';
+  $featuredArticlesXpath{el} = '/html/body/div[3]/div[3]/div[4]/table[3]/tr[2]/td/p/b/a/@href';
+  
+  #$featuredArticlesXpath{en} = '//span[@class="featured_article_metadata has_been_on_main_page"]/a/@href';
+  $featuredArticlesXpath{en} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td/p//a';
+  
+  #$featuredArticlesXpath{eo} = '/html/body/div/div/table/tr[position()>3]/td/p//a/@href';
+  $featuredArticlesXpath{eo} = '/html/body/div[3]/div[3]/div[4]/table/tr[position()>3]/td/p/a/@href';
+
+  #$featuredArticlesXpath{es} = '/html/body/div/div/table[3]/tr[3]/td//a/@href';
+  $featuredArticlesXpath{es} = '/html/body/div[3]/div[3]/div[4]/table[3]/tr[3]/td/p//a/@href';
+  
+  #$featuredArticlesXpath{eu} = '/html/body/div/div/ul/li/a/@href';
+  $featuredArticlesXpath{eu} = '/html/body/div[3]/div[3]/div[4]/p/a/@href';
+
+  #$featuredArticlesXpath{fa} = '/html/body/div/div/table[3]/tr[2]/td//a/@href';
+  $featuredArticlesXpath{fa} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td//a/@href';
+
+  #$featuredArticlesXpath{fi} = '/html/body/div/div/table[position()>3]//a/@href';
+  $featuredArticlesXpath{fi} = '/html/body/div[3]/div[3]/div[4]/table[4]/tr/td/p/a/@href';
+  
+  $featuredArticlesXpath{he} = \&_getFeaturedArticlesOfLanguageCodeHe;
+
+  #$featuredArticlesXpath{hr} = '/html/body/div/div/div/div[2]/table/tr[4]/td//a/@href';
+  $featuredArticlesXpath{hr} = '/html/body/div[3]/div[3]/div[4]/table/tr[4]/td/p/font//a/@href';
+  
+  #$featuredArticlesXpath{hu} = '/html/body/div/div/table[3]/tr/td/p//a/@href';
+  $featuredArticlesXpath{hu} = '/html/body/div[3]/div[3]/div[4]/table[3]/tr/td/p/a/@href';
+  
+  #$featuredArticlesXpath{it} = '/html/body/div/div/table/tr[7]/td/dl/dd//a/@href';
+  $featuredArticlesXpath{it} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr/td/table[2]/tr[6]/td/dl/dd//a/@href';
+  
+  #$featuredArticlesXpath{id} = '/html/body/div/div/table[2]/tr[2]/td//a/@href';
+  $featuredArticlesXpath{id} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr[2]/td/ul/li/div/div[2]/p/a/@href';
+
+  #$featuredArticlesXpath{ja} = '/html/body/div/div/ul/li//a/@href';
+  $featuredArticlesXpath{ja} = '/html/body/div[3]/div[3]/div[4]/ul[position()<10]/li//a/@href';
+
+  $featuredArticlesXpath{jv} = '/html/body/div[3]/div[3]/div[4]/ul/li/a/@href';
+ 
+  #$featuredArticlesXpath{ka} = '/html/body/div/div/div/table/tr[position()>2]//a/@href';
+  $featuredArticlesXpath{ka} = '/html/body/div[3]/div[3]/div[4]/div/table/tr/td/table[2]/tr/td/p/a/@href';
+  
+  $featuredArticlesXpath{kk} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr/td/table[4]/tr/td/p/a/@href';
+  
+  $featuredArticlesXpath{km} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td/ul/li//a/@href';
+  
+  #$featuredArticlesXpath{ko} = '/html/body/div/div/table/tr[3]/td//a/@href';
+  $featuredArticlesXpath{ko} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td/p/a/@href';
+  
+  $featuredArticlesXpath{li} = '/html/body/div[3]/div[3]/div[4]/ul[2]/li/a/@href';
+
+  $featuredArticlesXpath{lv} = '/html/body/div[3]/div[3]/div[4]/table/tr/td/table[3]/tr/td/p//a/@href';
+
+  #$featuredArticlesXpath{ml} = '/html/body/div/div/table/tr/td[2]//a/@href';
+  $featuredArticlesXpath{ml} = '/html/body/div[4]/div[3]/div[4]/table/tr/td[2]/a/@href';
+
+  #$featuredArticlesXpath{mr} = '//*[@id="bodyContent"]//b/a/@href';
+  $featuredArticlesXpath{mr} = '/html/body/div[3]/div[3]/div[4]/ul/li/b/a/@href';
+  
+  #$featuredArticlesXpath{ms} = '/html/body/div/div/table/tr[3]/td//a/@href';
+  $featuredArticlesXpath{ms} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td/p/span/a/@href';
+  
+  $featuredArticlesXpath{mzn} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr[2]/td/ul/li/b/a/@href';
+  
+  #$featuredArticlesXpath{nl} = '/html/body/div/div/table/tr/td/p//a/@href';
+  $featuredArticlesXpath{nl} = '/html/body/div[3]/div[3]/div[4]/table/tr[position()>2]/td/p//a/@href';
+  
+  #$featuredArticlesXpath{nn} = '/html/body/div/div/table/tr/td/table/tr/td/table/tr/td/ul/li/a/@href';
+  $featuredArticlesXpath{nn} = '/html/body/div[3]/div[3]/div[4]/table/tr[4]/td/table/tr/td/ul/li/a/@href';
+
+  #$featuredArticlesXpath{no} = '/html/body/div/div/table/tr[position()>1]/td//a/@href';
+  $featuredArticlesXpath{no} = '/html/body/div[3]/div[3]/div[4]/table/tr[position()>1]/td/table/tr/td/p//a/@href';
+
+  #$featuredArticlesXpath{pl} = '/html/body/div/div/table/tr[3]/td//a/@href';
+  $featuredArticlesXpath{pl} = '/html/body/div[3]/div[3]/div[4]/p[position()>1]/a/@href';
+  
+  #$featuredArticlesXpath{pt} = '/html/body/div/div/table/tr/td/dl/dd//a/@href';
+  $featuredArticlesXpath{pt} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr[3]/td/dl/dd//a/@href';
+
+  #$featuredArticlesXpath{ro} = '/html/body/div/div/table[2]/tr[3]/td/dl/dd//a/@href';
+  $featuredArticlesXpath{ro} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr[3]/td/dl/dd/a/@href';
+
+  $featuredArticlesXpath{ru} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr/td/table[4]/tr/td//a/@href';
+
+  #$featuredArticlesXpath{sh} = '/html/body/div/div/div/div/ul/li/b//a/@href';
+  $featuredArticlesXpath{sh} = '/html/body/div[3]/div[3]/div[4]/table/tr[4]/td/p/font/b/a/@href';
+
+  #$featuredArticlesXpath{sk} = '/html/body/div/div/p//a/@href';
+  $featuredArticlesXpath{sk} = '/html/body/div[3]/div[3]/div[4]/p/b//a/@href';
+  
+  $featuredArticlesXpath{simple} = '/html/body/div[3]/div[3]/div[4]/table[1]/tr/td[2]//a/@href';
+
+  #$featuredArticlesXpath{sl} = '/html/body/div/div/table[2]/tr[3]/td//a/@href';
+  $featuredArticlesXpath{sl} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr[3]/td/p//a/@href';
+  
+  #$featuredArticlesXpath{sr} = '/html/body/div/div/div/div/table/tr/td/dl/dd//a/@href';
+  $featuredArticlesXpath{sr} = '/html/body/div[3]/div[3]/div[4]/table[3]/tr[3]/td/dl/dd/a/@href';
+  
+  #$featuredArticlesXpath{sv} = '/html/body/div/div/table/tr/td/table/tr/td/dl/dd/b//a/@href';
+  $featuredArticlesXpath{sv} = '/html/body/div[3]/div[3]/div[4]/table/tr/td/table[3]/tr/td/dl/dd/a/@href';
+  
+  $featuredArticlesXpath{sw} = '/html/body/div[3]/div[3]/div[4]/div/div/div/a/@href';
+
+  $featuredArticlesXpath{ta} = '/html/body/div[3]/div[3]/div[4]/p/b/a/@href';
+  
+  $featuredArticlesXpath{th} = '/html/body/div[3]/div[3]/div[4]/table[3]/tr/td/ul/li//a/@href';
+  
+  $featuredArticlesXpath{tl} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td/p/a/@href';
+  
+  #$featuredArticlesXpath{tr} = '/html/body/div/div/table/tr[3]/td/p//a/@href';
+  $featuredArticlesXpath{tr} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td/p//a/@href';
+  
+  $featuredArticlesXpath{tt} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr/td/table/tr/td/table[3]/tr/td/table[2]/tr/td/p/a/@href|/html/body/div[3]/div[3]/div[4]/table[2]/tr/td/table/tr/td/table[3]/tr/td/table[2]/tr/td//ul/li/a';
+  
+  $featuredArticlesXpath{uk} = '/html/body/div[3]/div[3]/div[4]/table[2]/tr[2]/td//ul/li/a/@href';
+  
+  $featuredArticlesXpath{ur} = '/html/body/div[3]/div[3]/div[4]/table/tr[3]/td/p/a/@href';
+  
+  #$featuredArticlesXpath{vi} = '/html/body/div/div/div/table/tr/td/p//a/@href';
+  $featuredArticlesXpath{vi} = '/html/body/div[3]/div[3]/div[4]/div[2]/table/tr/td[2]/p//a/@href';
+  
+  $featuredArticlesXpath{vo} = '/html/body/div[3]/div[3]/div[4]/div/div/div/a';
+  
+  #$featuredArticlesXpath{zh} = '/html/body/div/div/table[3]//a/@href';
+  $featuredArticlesXpath{zh} = '/html/body/div[3]/div[3]/div[4]/table[3]/tr/td/p/b/a/@href';
 
   # multi-page featured article wikipedias require a subroutine to extract the featured article urls.
   $featuredArticlesXpath{fr} = \&_getFeaturedArticlesOfLanguageCodeFr;
-  $featuredArticlesXpath{he} = \&_getFeaturedArticlesOfLanguageCodeHe;
 
   $_[0]->{featuredArticlesXpath} = \%featuredArticlesXpath if @_;
   return \%featuredArticlesXpath;
@@ -1465,9 +1650,9 @@ sub _getFeaturedArticlesOfLanguageCodeFr
     close $filePtr;
 
     # get the list of featured article urls.
-    my @listOfUrls = $htmlParser->findvalues ('/html/body/div/div/div/table[2]/tr/td/ul/li/a/@href');
+    my @listOfUrls = $htmlParser->findvalues ('/html/body/div[3]/div[3]/div[4]/div[3]/div[2]/div/table/tr/td/ul/li/a/@href');
 
-    # log a warning if not urls extracted.
+    # log a warning if no urls extracted.
     unless (@listOfUrls)
     {
       my $logger = Log::Log4perl->get_logger();
@@ -1492,12 +1677,12 @@ sub _getFeaturedArticlesOfLanguageCodeFr
 
     # if there are a lot of repeats there are parsing problems.
 
-    my @potentialNextNodes = $htmlParser->findnodes ('/html/body/div/div/a');
+    my @potentialNextNodes = $htmlParser->findnodes ('/html/body//a');
     my $nextUrl;
     foreach my $node (@potentialNextNodes)
     {
       my $textOfNode = lc $node->findvalue ('.');
-      if (index ($textOfNode, 'suivants') > -1)
+      if (index ($textOfNode, 'ments suivants') > -1)
       {
         $nextUrl = $node->findvalue ('@href');
         last;
@@ -1511,7 +1696,7 @@ sub _getFeaturedArticlesOfLanguageCodeFr
     $url = $nextUrl;
 
     # make sure the loop stops.
-    if ($pageIndex > 20)
+    if ($pageIndex > 40)
     {
       my $logger = Log::Log4perl->get_logger();
       my $languageCode = $Self->{languageCode};
@@ -1560,7 +1745,7 @@ sub _getFeaturedArticlesOfLanguageCodeHe
     close $filePtr;
 
     # get the list of featured article urls.
-    my @listOfUrls = $htmlParser->findvalues ('/html/body/div/div/div/table/tr/td/ul/li/a/@href');
+    my @listOfUrls = $htmlParser->findvalues ('/html/body/div[3]/div[3]/div[4]/div/div/div/table/tr/td/ul/li/a/@href');
 
     # log a warning if no urls extracted.
     unless (@listOfUrls)
@@ -1586,14 +1771,14 @@ sub _getFeaturedArticlesOfLanguageCodeHe
     @listOfUrls = ();
 
     # if there are a lot of repeats there are parsing problems.
-    my @potentialNextNodes = $htmlParser->findnodes ('/html/body/div[3]/div/a');
+    my @potentialNextNodes = $htmlParser->findnodes ('/html/body/div[3]/div[3]/div[4]/div/div/a[2]');
 
     my $nextUrl;
     foreach my $node (@potentialNextNodes)
     {
       my $textOfNode = lc $node->findvalue ('.');
       my $nextPageLinkText = pack("H*","32303020d794d791d790d799d79d");
-      if ($textOfNode =~ /$nextPageLinkText/i)
+      if (($textOfNode =~ /$nextPageLinkText/i) || ($textOfNode =~ /הבאים/i))
       {
         $nextUrl = $node->findvalue ('@href');
         last;
@@ -1707,10 +1892,10 @@ sub _setWikipediaLanguageCodes
   my $Self = $_[0];
 
   # set the url of the page of all wikipeida languages
-  my $url = 'http://meta.wikimedia.org/wiki/List_of_Wikipedias_by_language_family';
+  my $url = 'http://meta.wikimedia.org/wiki/List_of_Wikipedias';
 
   # build the full pathname of the file to save the page to.
-  my $outputFile = File::Spec->catfile ($Self->{featuredDirectory}, 'List_of_Wikipedias_by_language_family.html');
+  my $outputFile = File::Spec->catfile ($Self->{featuredDirectory}, 'List_of_Wikipedias.html');
 
   # only fetch the file if it is too old.
   my $fetchFile = $Self->_isFileTooOld (filePath => $outputFile, maxAge => 24 * 60 * 60);
@@ -1733,21 +1918,27 @@ sub _setWikipediaLanguageCodes
 
   # extract out the language info of each wikipedia language listed.
   # my $xpath = '/html/body/div/div/div/div[2]/ol/li';
-  my $xpath = '/html/body/div/div/ol/li';
-  my @listOfLanguageInfo = $htmlParser->findvalues ($xpath);
+  my $xpath = '/html/body/div[3]/div[3]/div[4]/table/tr';
+  my @listOfRows = $htmlParser->findnodes ($xpath);
 
   # log a warning if no language found.
-  unless (@listOfLanguageInfo)
+  unless (@listOfRows)
   {
     my $logger = Log::Log4perl->get_logger();
     $logger->logwarn ("Could not extract corresponding English name of the language codes from page '$url'; format change my require a new xpath expression.\n");
   }
 
   my %enNameOfLanguageCode;
-  foreach my $languageInfo (@listOfLanguageInfo)
+  foreach my $languageInfo (@listOfRows)
   {
-    my $enLanguageName = $1 if ($languageInfo =~ /^([^ \(]+)/);
-    my $languageCode = $1 if ($languageInfo =~ / ([^ ]+) [^ ]+ [\d\,]+ articles/);
+    # get each column node.
+    my @columns = $languageInfo->findnodes ('./td');
+    
+    # skip the row if it does not have 12 columns.
+    next unless @columns == 12;
+    
+    my $enLanguageName = $languageInfo->findvalue ('./td[2]');
+    my $languageCode = $languageInfo->findvalue ('./td[4]');
     $enNameOfLanguageCode{$languageCode} = $enLanguageName if (defined ($enLanguageName) && defined ($languageCode));
   }
   $Self->{enNameOfLanguageCode} = \%enNameOfLanguageCode;
@@ -1789,7 +1980,7 @@ sub _setWikipediaFeatureArticleInfo
 
   # extract out the language info of each wikipedia language listed.
   # my $xpath = '/html/body/div/div/div/div[2]/table/tr';
-  my $xpath = '/html/body/div/div/table/tr';
+  my $xpath = '/html/body/div[3]/div[3]/div[4]/table/tr';
   my @listOfLanguageInfo = $htmlParser->findnodes ($xpath);
 
   # log a warning if no language info found.
@@ -1803,8 +1994,11 @@ sub _setWikipediaFeatureArticleInfo
   my %langCodeFeatureArticlesInfo;
   foreach my $languageInfo (@listOfLanguageInfo)
   {
-    my $url = lc $languageInfo->findvalue ('td[1]/a/@href');
-    my $languageCode = $1 if ($url =~ /^http:\/\/([^\.]+)/);
+    my $url;
+    $url = lc $languageInfo->findvalue ('td[1]/a/@href');
+    next unless (defined ($url) && length ($url));
+    my $languageCode;
+    $languageCode = $2 if ($url =~ /^(http:)?\/\/([^\.]+)/);
     next unless defined $languageCode;
 
     my $name = $languageInfo->findvalue ('td[1]');
@@ -1829,6 +2023,7 @@ sub _setWikipediaFeatureArticleInfo
 The XML files are created using L<XML::Code> with the simple structure outlined below:
 
   <document>
+    <title>The Article Title</title>
     <summary>
       <p>This is the first paragraph of the summary.</p>
       <p>This is the second paragraph of the summary.</p>
@@ -1890,14 +2085,10 @@ using this module.
 
 =head1 INSTALLATION
 
-To install the module run the following commands:
+Use L<CPAN> to install the module and all its prerequisites:
 
-  perl Makefile.PL
-  make
-  make test
-  make install
-
-If you are on a windows box you should use 'nmake' rather than 'make'.
+  perl -MCPAN -e shell
+  >install Text::Corpus::Summaries::Wikipedia
 
 =head1 BUGS
 
@@ -1936,7 +2127,9 @@ L<create_summary_corpus.pl>, L<Encode>, L<Forks::Super>, L<HTML::TreeBuilder::XP
 Links to the featured article page for the supported language codes:
 <a href="http://af.wikipedia.org/wiki/Wikipedia:Voorbladartikel">af:Afrikaans</a>,
 <a href="http://ar.wikipedia.org/wiki/%D9%88%D9%8A%D9%83%D9%8A%D8%A8%D9%8A%D8%AF%D9%8A%D8%A7:%D9%85%D9%82%D8%A7%D9%84%D8%A7%D8%AA_%D9%85%D8%AE%D8%AA%D8%A7%D8%B1%D8%A9">ar:Arabic</a>,
+<a href="http://az.wikipedia.org/wiki/Vikipediya:Se%C3%A7ilmi%C5%9F_m%C9%99qal%C9%99l%C9%99r">az:Azerbaijani</a>,
 <a href="http://bg.wikipedia.org/wiki/%D0%A3%D0%B8%D0%BA%D0%B8%D0%BF%D0%B5%D0%B4%D0%B8%D1%8F:%D0%98%D0%B7%D0%B1%D1%80%D0%B0%D0%BD%D0%B8_%D1%81%D1%82%D0%B0%D1%82%D0%B8%D0%B8">bg:Bulgarian</a>,
+<a href="http://bs.wikipedia.org/wiki/Wikipedia:Odabrani_%C4%8Dlanci">bs:Bosnian</a>,
 <a href="http://ca.wikipedia.org/wiki/Viquip%C3%A8dia:Articles_de_qualitat">ca:Catalan</a>,
 <a href="http://cs.wikipedia.org/wiki/Wikipedie:Nejlep%C5%A1%C3%AD_%C4%8Dl%C3%A1nky">cs:Czech</a>,
 <a href="http://de.wikipedia.org/wiki/Wikipedia:Exzellente_Artikel">de:German</a>,
@@ -1945,35 +2138,55 @@ Links to the featured article page for the supported language codes:
 <a href="http://eo.wikipedia.org/wiki/Vikipedio:Elstaraj_artikoloj">eo:Esperanto</a>,
 <a href="http://es.wikipedia.org/wiki/Wikipedia:Art%C3%ADculos_destacados">es:Spanish</a>,
 <a href="http://eu.wikipedia.org/wiki/Wikipedia:Nabarmendutako_artikuluak">eu:Basque</a>,
-<a href="http://fa.wikipedia.org/wiki/%D9%88%DB%8C%DA%A9%DB%8C%E2%80%8C%D9%BE%D8%AF%DB%8C%D8%A7:%D9%86%D9%88%D8%B4%D8%AA%D8%A7%D8%B1%D9%87%D8%A7%DB%8C_%D8%A8%D8%B1%DA%AF%D8%B2%DB%8C%D8%AF%D9%87">fa:Persian</a>,
-<a href="http://fi.wikipedia.org/wiki/Wikipedia:Suositellut_sivut">fi:Finnish</a>,
+<a href="http://fa.wikipedia.org/wiki/%D9%88%DB%8C%DA%A9%DB%8C%E2%80%8C%D9%BE%D8%AF%DB%8C%D8%A7:%D9%85%D9%82%D8%A7%D9%84%D9%87%E2%80%8C%D9%87%D8%A7%DB%8C_%D8%A8%D8%B1%DA%AF%D8%B2%DB%8C%D8%AF%D9%87">fa:Persian</a>,
+<a href="http://fi.wikipedia.org/wiki/Wikipedia:Suositellut_artikkelit">fi:Finnish</a>,
 <a href="http://fr.wikipedia.org/wiki/Wikip%C3%A9dia:Articles_de_qualit%C3%A9">fr:French</a>,
 <a href="http://he.wikipedia.org/wiki/%D7%A4%D7%95%D7%A8%D7%98%D7%9C:%D7%A2%D7%A8%D7%9B%D7%99%D7%9D_%D7%9E%D7%95%D7%9E%D7%9C%D7%A6%D7%99%D7%9D">he:Hebrew</a>,
 <a href="http://hr.wikipedia.org/wiki/Wikipedija:Izabrani_%C4%8Dlanci">hr:Croatian</a>,
-<a href="http://hu.wikipedia.org/wiki/Wikip%C3%A9dia:Kiemelt_sz%C3%B3cikkek">hu:Hungarian</a>,
+<a href="http://hu.wikipedia.org/wiki/Wikip%C3%A9dia:Kiemelt_sz%C3%B3cikkek_bemutat%C3%B3ja">hu:Hungarian</a>,
 <a href="http://id.wikipedia.org/wiki/Wikipedia:Artikel_pilihan/Topik">id:Indonesian</a>,
 <a href="http://it.wikipedia.org/wiki/Wikipedia:Vetrina">it:Italian</a>,
 <a href="http://ja.wikipedia.org/wiki/Wikipedia:%E7%A7%80%E9%80%B8%E3%81%AA%E8%A8%98%E4%BA%8B">ja:Japanese</a>,
+<a href="http://jv.wikipedia.org/wiki/Wikipedia:Artikel_pilihan">jv:Javanese</a>,
 <a href="http://ka.wikipedia.org/wiki/%E1%83%95%E1%83%98%E1%83%99%E1%83%98%E1%83%9E%E1%83%94%E1%83%93%E1%83%98%E1%83%90:%E1%83%A0%E1%83%A9%E1%83%94%E1%83%A3%E1%83%9A%E1%83%98_%E1%83%A1%E1%83%A2%E1%83%90%E1%83%A2%E1%83%98%E1%83%94%E1%83%91%E1%83%98">ka:Georgian</a>,
+<a href="http://kk.wikipedia.org/wiki/%D0%A3%D0%B8%D0%BA%D0%B8%D0%BF%D0%B5%D0%B4%D0%B8%D1%8F:%D0%A2%D0%B0%D2%A3%D0%B4%D0%B0%D1%83%D0%BB%D1%8B_%D0%BC%D0%B0%D2%9B%D0%B0%D0%BB%D0%B0%D0%BB%D0%B0%D1%80">kk:Kazakh</a>,
+<a href="http://km.wikipedia.org/wiki/%E1%9E%9C%E1%9E%B7%E1%9E%82%E1%9E%B8%E1%9E%97%E1%9E%B8%E1%9E%8C%E1%9E%B6:%E1%9E%A2%E1%9E%8F%E1%9F%92%E1%9E%90%E1%9E%94%E1%9E%91%E1%9E%96%E1%9E%B7%E1%9E%9F%E1%9F%81%E1%9E%9F">km:Khmer</a>,
 <a href="http://ko.wikipedia.org/wiki/%EC%9C%84%ED%82%A4%EB%B0%B1%EA%B3%BC:%EC%95%8C%EC%B0%AC_%EA%B8%80">ko:Korean</a>,
+<a href="http://li.wikipedia.org/wiki/Wikipedia:Sjterartikel">li:Limburgish</a>,
+<a href="http://lv.wikipedia.org/wiki/Vikip%C4%93dija:V%C4%93rt%C4%ABgi_raksti">lv:Latvian</a>,
 <a href="http://ml.wikipedia.org/wiki/%E0%B4%B5%E0%B4%BF%E0%B4%95%E0%B5%8D%E0%B4%95%E0%B4%BF%E0%B4%AA%E0%B5%80%E0%B4%A1%E0%B4%BF%E0%B4%AF:%E0%B4%A4%E0%B4%BF%E0%B4%B0%E0%B4%9E%E0%B5%8D%E0%B4%9E%E0%B5%86%E0%B4%9F%E0%B5%81%E0%B4%A4%E0%B5%8D%E0%B4%A4_%E0%B4%B2%E0%B5%87%E0%B4%96%E0%B4%A8%E0%B4%99%E0%B5%8D%E0%B4%99%E0%B4%B3%E0%B5%8D%E2%80%8D">ml:Malayalam</a>,
 <a href="http://mr.wikipedia.org/wiki/%E0%A4%B5%E0%A4%BF%E0%A4%95%E0%A4%BF%E0%A4%AA%E0%A5%80%E0%A4%A1%E0%A4%BF%E0%A4%AF%E0%A4%BE:%E0%A4%AE%E0%A4%BE%E0%A4%B8%E0%A4%BF%E0%A4%95_%E0%A4%B8%E0%A4%A6%E0%A4%B0/%E0%A4%AE%E0%A4%BE%E0%A4%97%E0%A5%80%E0%A4%B2_%E0%A4%85%E0%A4%82%E0%A4%95_%E0%A4%B8%E0%A4%82%E0%A4%97%E0%A5%8D%E0%A4%B0%E0%A4%B9">mr:Marathi</a>,
 <a href="http://ms.wikipedia.org/wiki/Wikipedia:Rencana_pilihan">ms:Malay</a>,
+<a href="http://mzn.wikipedia.org/wiki/%D9%88%DB%8C%DA%A9%DB%8C%E2%80%8C%D9%BE%D8%AF%DB%8C%D8%A7:%D8%AE%D8%A7%D8%B1_%D8%A8%D9%86%D9%88%DB%8C%D8%B4%D8%AA%D9%87">mzn:Mazandarani</a>,
 <a href="http://nl.wikipedia.org/wiki/Wikipedia:Etalage">nl:Dutch</a>,
-<a href="http://nn.wikipedia.org/wiki/Wikipedia:Gode_artiklar">nn:Norwegian</a>,
-<a href="http://no.wikipedia.org/wiki/Wikipedia:Utmerkede_artikler">no:Norwegian</a>,
+<a href="http://nn.wikipedia.org/wiki/Wikipedia:Gode_artiklar">nn:Norwegian (Nynorsk)</a>,
+<a href="http://no.wikipedia.org/wiki/Wikipedia:Utmerkede_artikler">no:Norwegian (Bokm?l)</a>,
 <a href="http://pl.wikipedia.org/wiki/Wikipedia:Artyku%C5%82y_na_medal">pl:Polish</a>,
 <a href="http://pt.wikipedia.org/wiki/Wikipedia:Artigos_destacados">pt:Portuguese</a>,
 <a href="http://ro.wikipedia.org/wiki/Wikipedia:Articole_de_calitate">ro:Romanian</a>,
-<a href="http://sh.wikipedia.org/wiki/Wikipedia:Izabrani_%C4%8Dlanci">sh:Serbocroatian</a>,
-<a href="http://sk.wikipedia.org/wiki/Wikip%C3%A9dia:Najlep%C5%A1ie_%C4%8Dl%C3%A1nky">sk:Slovak</a>,
+<a href="http://ru.wikipedia.org/wiki/%D0%92%D0%B8%D0%BA%D0%B8%D0%BF%D0%B5%D0%B4%D0%B8%D1%8F:%D0%98%D0%B7%D0%B1%D1%80%D0%B0%D0%BD%D0%BD%D1%8B%D0%B5_%D1%81%D1%82%D0%B0%D1%82%D1%8C%D0%B8">ru:Russian</a>,
+<a href="http://sh.wikipedia.org/wiki/Wikipedia:Izabrani_%C4%8Dlanci">sh:Serbo-Croatian</a>,
+<a href="http://simple.wikipedia.org/wiki/Wikipedia:Very_good_articles/by_date">simple:Simple English</a>,
+<a href="http://sk.wikipedia.org/wiki/Wikip%C3%A9dia:Zoznam_najlep%C5%A1%C3%ADch_%C4%8Dl%C3%A1nkov">sk:Slovak</a>,
 <a href="http://sl.wikipedia.org/wiki/Wikipedija:Izbrani_%C4%8Dlanki">sl:Slovenian</a>,
 <a href="http://sr.wikipedia.org/wiki/%D0%92%D0%B8%D0%BA%D0%B8%D0%BF%D0%B5%D0%B4%D0%B8%D1%98%D0%B0:%D0%A1%D1%98%D0%B0%D1%98%D0%BD%D0%B8_%D1%82%D0%B5%D0%BA%D1%81%D1%82%D0%BE%D0%B2%D0%B8">sr:Serbian</a>,
 <a href="http://sv.wikipedia.org/wiki/Wikipedia:Utm%C3%A4rkta_artiklar">sv:Swedish</a>,
+<a href="http://sw.wikipedia.org/wiki/Wikipedia:Featured_articles">sw:Swahili</a>,
+<a href="http://ta.wikipedia.org/wiki/%E0%AE%B5%E0%AE%BF%E0%AE%95%E0%AF%8D%E0%AE%95%E0%AE%BF%E0%AE%AA%E0%AF%8D%E0%AE%AA%E0%AF%80%E0%AE%9F%E0%AE%BF%E0%AE%AF%E0%AE%BE:%E0%AE%9A%E0%AE%BF%E0%AE%B1%E0%AE%AA%E0%AF%8D%E0%AE%AA%E0%AF%81%E0%AE%95%E0%AF%8D_%E0%AE%95%E0%AE%9F%E0%AF%8D%E0%AE%9F%E0%AF%81%E0%AE%B0%E0%AF%88%E0%AE%95%E0%AE%B3%E0%AF%8D">ta:Tamil</a>,
 <a href="http://th.wikipedia.org/wiki/%E0%B8%A7%E0%B8%B4%E0%B8%81%E0%B8%B4%E0%B8%9E%E0%B8%B5%E0%B9%80%E0%B8%94%E0%B8%B5%E0%B8%A2:%E0%B8%9A%E0%B8%97%E0%B8%84%E0%B8%A7%E0%B8%B2%E0%B8%A1%E0%B8%84%E0%B8%B1%E0%B8%94%E0%B8%AA%E0%B8%A3%E0%B8%A3">th:Thai</a>,
+<a href="http://tl.wikipedia.org/wiki/Wikipedia:Mga_napiling_artikulo">tl:Tagalog</a>,
 <a href="http://tr.wikipedia.org/wiki/Vikipedi:Se%C3%A7kin_maddeler">tr:Turkish</a>,
-<a href="http://vi.wikipedia.org/wiki/Wikipedia:B%C3%A0i_vi%E1%BA%BFt_ch%E1%BB%8Dn_l%E1%BB%8Dc">vi:Vietnamese</a>, and
-<a href="http://zh.wikipedia.org/wiki/Wikipedia:%E7%89%B9%E8%89%B2%E6%9D%A1%E7%9B%AE">zh:Chinese</a>.</p>
+<a href="http://tt.wikipedia.org/wiki/%D0%92%D0%B8%D0%BA%D0%B8%D0%BF%D0%B5%D0%B4%D0%B8%D1%8F:%D0%A1%D0%B0%D0%B9%D0%BB%D0%B0%D0%BD%D0%B3%D0%B0%D0%BD_%D0%BC%D3%99%D0%BA%D0%B0%D0%BB%D3%99%D0%BB%D3%99%D1%80">tt:Tatar</a>,
+<a href="http://uk.wikipedia.org/wiki/%D0%92%D1%96%D0%BA%D1%96%D0%BF%D0%B5%D0%B4%D1%96%D1%8F:%D0%92%D0%B8%D0%B1%D1%80%D0%B0%D0%BD%D1%96_%D1%81%D1%82%D0%B0%D1%82%D1%82%D1%96">uk:Ukrainian</a>,
+<a href="http://ur.wikipedia.org/wiki/%D9%85%D9%86%D8%B5%D9%88%D8%A8%DB%81:%D9%85%D9%86%D8%AA%D8%AE%D8%A8_%D9%85%D8%B6%D9%85%D9%88%D9%86">ur:Urdu</a>,
+<a href="http://vi.wikipedia.org/wiki/Wikipedia:B%C3%A0i_vi%E1%BA%BFt_ch%E1%BB%8Dn_l%E1%BB%8Dc">vi:Vietnamese</a>,
+<a href="http://vo.wikipedia.org/wiki/V%C3%BCkiped:Yegeds_gudik">vo:Volap?k</a>, and
+<a href="http://zh.wikipedia.org/wiki/Wikipedia:%E7%89%B9%E8%89%B2%E6%9D%A1%E7%9B%AE">zh:Chinese</a>.
+</p>
+
+<p>
+Copies of the data sets generated in May 2010 and February 2013 can be download <a href="http://jeffkubina.org/data/wfa">here</a>.
+</p>
 
 =end html
 
